const {Parcel} = require('@parcel/core');

let bundler = new Parcel({
  entries: 'index.html',
  defaultConfig: '@parcel/config-default',
  mode: 'production',
	serveOptions: {
    port: 3000
  },
  hmrOptions: {
    port: 3000
  },
  defaultTargetOptions: {
    engines: {
      browsers: ['last 1 Chrome version']
    }
  }
});

const run = async () => {
	try {
		let {bundleGraph, buildTime} = await bundler.run();
		let bundles = bundleGraph.getBundles();
		console.log(`✨ Built ${bundles.length} bundles in ${buildTime}ms!`);
	} catch (err) {
		console.log(err.diagnostics);
	}
}

const watch = async () => {
	let subscription = await bundler.watch((err, event) => {
		if (err) {
			// fatal error
			throw err;
		}

		if (event.type === 'buildSuccess') {
			let bundles = event.bundleGraph.getBundles();
			console.log(`✨ Built ${bundles.length} bundles in ${event.buildTime}ms!`);
			console.log('\x1b[36m%s\x1b[0m', ' Your app is ready to go on http://localhost:3000');  //cyan
		} else if (event.type === 'buildFailure') {
			console.log(event.diagnostics);
		}
	});
}

// run();
watch();
