import { Group, paper, Path, Point, PointText, Tool } from "paper";
import { rad2deg } from "../utils/convertions";
import { Pane } from 'tweakpane';
import {} from '../utils/arrayOperations'
import "../styles.css";
import random from 'canvas-sketch-util/random';
import { Color } from "paper/dist/paper-core";

const canvas = document.getElementById("paperCanvas");

paper.install(window);
paper.setup(canvas);

const cols = 32;
const rows = 24;
const numOfCells = cols * rows;

const gridWidth = view.size.width * 0.8;
const gridHeight = view.size.height * 0.8;

const cellWidth = gridWidth / cols;
const cellHeight = gridHeight / rows;

const margX = (view.size.width - gridWidth) * 0.5;
const margY = (view.size.height - gridHeight) * 0.5;

paper.project.view.onFrame = onFrame;

function onFrame(event) {
  if(event.count % 6 !== 0) return;
  paper.project.activeLayer.removeChildren();
  console.log('frame')
  for(let i=0; i<numOfCells ; i++)
  {
    const col = i % cols;
    const row = Math.floor(i / cols);

    const w = cellWidth * 0.8;
    const h = cellHeight * 0.8;
    const x = col * cellWidth + margX + (cellWidth - w) / 2;
    const y = row * cellHeight + margY + (cellHeight - h) / 2;
    
    const rand = random.noise2D(x + event.count * 2, y, 0.002) * Math.PI;
    // console.log(rand)
    
    // const rect = new Path.Rectangle({
    //   point: [x, y],
    //   size: [w, rand/Math.PI *  h],
    //   fillColor: 'black',
    //   strokeWidth: 1,
    // });

    const rect = new Path.Circle({
      center: [x, y],
      radius: rand/Math.PI * h * 0.75,
      fillColor: new Color(Math.abs(rand/Math.PI * 0.75)),
      strokeWidth: 1,
    });

    rect.rotate(rad2deg(rand));
  }
}

