/** @see http://paperjs.org/tutorials/geometry/vector-geometry/ */

import { Group, paper, Path, Point, PointText, Tool } from "paper";
import { rad2deg } from "../utils/convertions";
import { Pane } from 'tweakpane';
import {} from '../utils/arrayOperations'

console.log([1,2,3,4].shuffle())

import "../styles.css";

const canvas = document.getElementById("paperCanvas");

// const tool = new Tool();

paper.install(window);
paper.setup(canvas);
// tool.onMouseDown = onMouseDown;
// tool.onMouseDrag = onMouseDrag;
// tool.onMouseUp = onMouseUp;

const pane = new Pane();
const PARAMS = {
  balls: 30,
  color: '#ff0055',
  radius: 30
};

pane.addInput(PARAMS, 'balls');
const colorHandler = pane.addInput(PARAMS, 'color');
const inputHandler = pane.addInput(
  PARAMS, 'radius',
  {min: 12, max: 99, step: 3}
);

paper.project.view.onFrame = onFrame;
paper.project.currentStyle = {
	fillColor: PARAMS.color,
};

var numOfBalls = 30;
var range = (min, max) => Math.random() * (max - min) + min;

var ballPositions = [];

for(var i=0; i<numOfBalls; i++){
    var position = new Point(range(0, view.size.width), range(0, view.size.height));
    var velocity = new Point(range(-1, 1), range(-1, 1))
    ballPositions.push({position: position, velocity: velocity})
}

var handle_len_rate = 2.4;
var circlePaths = [];

for (var i = 0, l = ballPositions.length; i < l; i++) {
	var circlePath = new Path.Circle({
		center: ballPositions[i].position,
		radius: PARAMS.radius,
	});
	circlePaths.push(circlePath);
}

const reDraw = () => {
  const positions = [];
  for (var i = 0, l = circlePaths.length; i < l; i++) {
    positions.push(circlePaths[i].position)
    circlePaths[i].remove();
  }
  circlePaths = []
  for (var i = 0, l = positions.length; i < l; i++) {
    var circlePath = new Path.Circle({
      center: positions[i],
      radius: PARAMS.radius,
      fillColor: PARAMS.color
    });
    circlePaths.push(circlePath);
  }
}

colorHandler.on('change', function(ev) {
  reDraw();
});


inputHandler.on('change', function(ev) {
  reDraw();
});

function onFrame(event){
  if(event.count % 4 == 0){
    for (var i = 0, l = circlePaths.length; i < l; i++) {
        circlePaths[i].position = circlePaths[i].position.add(ballPositions[i].velocity);
        generateConnections(circlePaths);
        
        if(circlePaths[i].position.x < 0) circlePaths[i].position.x = view.size.width
        if(circlePaths[i].position.y < 0) circlePaths[i].position.y = view.size.height
        
        if(circlePaths[i].position.x > view.size.width) circlePaths[i].position.x = 0
        if(circlePaths[i].position.y > view.size.height) circlePaths[i].position.y = 0
    }
  }
}

var connections = new Group();
function generateConnections(paths) {
	// Remove the last connection paths:
	connections.children = [];

	for (var i = 0, l = paths.length; i < l; i++) {
		for (var j = i - 1; j >= 0; j--) {
			var path = metaball(paths[i], paths[j], .6, handle_len_rate, 200);
			if (path) 
				connections.appendTop(path);
		}
	}
}

generateConnections(circlePaths);

// ---------------------------------------------
function metaball(ball1, ball2, v, handle_len_rate, maxDistance) {
	var center1 = ball1.position;
	var center2 = ball2.position;
	var radius1 = ball1.bounds.width / 2;
	var radius2 = ball2.bounds.width / 2;
	var pi2 = Math.PI / 2;
	var d = center1.getDistance(center2);
	var u1, u2;

	if (radius1 == 0 || radius2 == 0) return;
	if (d > maxDistance || d <= Math.abs(radius1 - radius2)) return;
		
	if (d < radius1 + radius2) { // case circles are overlapping
		u1 = Math.acos((radius1 * radius1 + d * d - radius2 * radius2) /
				(2 * radius1 * d));
		u2 = Math.acos((radius2 * radius2 + d * d - radius1 * radius1) /
				(2 * radius2 * d));
	} else {
		u1 = 0;
		u2 = 0;
	}

	var angle1 = center2.subtract(center1).angleInRadians; //
	var angle2 = Math.acos((radius1 - radius2) / d);
	var angle1a = angle1 + u1 + (angle2 - u1) * v;
	var angle1b = angle1 - u1 - (angle2 - u1) * v;
	var angle2a = angle1 + Math.PI - u2 - (Math.PI - u2 - angle2) * v;
	var angle2b = angle1 - Math.PI + u2 + (Math.PI - u2 - angle2) * v;
	var p1a = center1.add(getVector(angle1a, radius1));
	var p1b = center1.add(getVector(angle1b, radius1));
	var p2a = center2.add(getVector(angle2a, radius2));
	var p2b = center2.add(getVector(angle2b, radius2));

	// define handle length by the distance between
	// both ends of the curve to draw
	var totalRadius = (radius1 + radius2);
	var d2 = Math.min(v * handle_len_rate, (p1a.subtract(p2a)).length / totalRadius);

	// case circles are overlapping:
	d2 *= Math.min(1, d * 2 / (radius1 + radius2));

	radius1 *= d2;
	radius2 *= d2;

	var path = new Path({
		segments: [p1a, p2a, p2b, p1b],
		style: ball1.style,
		closed: true
	});
	var segments = path.segments;

	segments[0].handleOut = getVector(angle1a - pi2, radius1);
	segments[1].handleIn = getVector(angle2a + pi2, radius2);
	segments[2].handleOut = getVector(angle2b - pi2, radius2);
	segments[3].handleIn = getVector(angle1b + pi2, radius1);
// 	path.fullySelected = true;
	return path;
}

// ------------------------------------------------
function getVector(radians, length) {
	return new Point({
		// Convert radians to degrees:
		angle: rad2deg(radians),
		length: length
	});
}